@@ -120,7 +120,6 @@ SessionController::SessionController(Session* sessionParam, TerminalDisplay* vie
     , _showMenuAction(nullptr)
     , _bookmarkValidProgramsToClear(QStringList())
     , _isSearchBarEnabled(false)
-    , _editProfileDialog(nullptr)
     , _searchBar(viewParam->searchBar())
     , _monitorProcessFinish(false)
     , _escapedUrlFilter(nullptr)
@@ -129,6 +128,7 @@ SessionController::SessionController(Session* sessionParam, TerminalDisplay* vie
     Q_ASSERT(viewParam);
 
     _sessionDisplayConnection = new SessionDisplayConnection(sessionParam, viewParam, this);
+    viewParam->setSessionController(this);
 
     // handle user interface related to session (menus etc.)
     if (isKonsolePart()) {
@@ -154,7 +154,6 @@ SessionController::SessionController(Session* sessionParam, TerminalDisplay* vie
     sessionAttributeChanged();
 
     connect(view(), &TerminalDisplay::compositeFocusChanged, this, &SessionController::viewFocusChangeHandler);
-    view()->setSessionController(this);
 
     // install filter on the view to highlight URLs and files
     updateFilterList(SessionManager::instance()->sessionProfile(session()));
@@ -248,9 +247,6 @@ SessionController::~SessionController()
 {
     _allControllers.remove(this);
 
-    if (!_editProfileDialog.isNull()) {
-        _editProfileDialog->deleteLater();
-    }
     if(factory() != nullptr) {
         factory()->removeClient(this);
     }
@@ -900,30 +896,10 @@ void SessionController::changeCodec(QTextCodec* codec)
     session()->setCodec(codec);
 }
 
-EditProfileDialog* SessionController::profileDialogPointer()
-{
-    return _editProfileDialog.data();
-}
-
 void SessionController::editCurrentProfile()
 {
-    // Searching for Edit profile dialog opened with the same profile
-    for (SessionController *controller : qAsConst(_allControllers)) {
-        if ( (controller->profileDialogPointer() != nullptr)
-             && controller->profileDialogPointer()->isVisible()
-             && (controller->profileDialogPointer()->lookupProfile()
-                 == SessionManager::instance()->sessionProfile(session())) ) {
-            controller->profileDialogPointer()->close();
-        }
-    }
-
-    // NOTE bug311270: For to prevent the crash, the profile must be reset.
-    if (!_editProfileDialog.isNull()) {
-        // exists but not visible
-        _editProfileDialog->deleteLater();
-    }
-
     auto profile = SessionManager::instance()->sessionProfile(session());
+    auto state = EditProfileDialog::ExistingProfile;
     // Don't edit the Fallback profile, instead create a new one
     if (profile->isFallback()) {
         auto newProfile = Profile::Ptr(new Profile(profile));
@@ -933,26 +909,31 @@ void SessionController::editCurrentProfile()
         newProfile->setProperty(Profile::UntranslatedName, uniqueName);
         profile = newProfile;
         SessionManager::instance()->setSessionProfile(session(), profile);
+        state = EditProfileDialog::NewProfile;
     }
 
-    _editProfileDialog = new EditProfileDialog(QApplication::activeWindow());
-    _editProfileDialog->setProfile(profile);
+    auto *dialog = new EditProfileDialog(QApplication::activeWindow());
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->setModal(true);
+    dialog->setProfile(profile, state);
 
-    connect(_editProfileDialog, &QDialog::accepted, this, [profile]() {
+    connect(dialog, &QDialog::accepted, this, [profile]() {
         ProfileManager::instance()->addProfile(profile);
         ProfileManager::instance()->changeProfile(profile, profile->setProperties());
     });
 
-    _editProfileDialog->show();
+    dialog->show();
 }
 
 void SessionController::renameSession()
 {
-    const QString &sessionLocalTabTitleFormat = session()->tabTitleFormat(Session::LocalTabTitle);
-    const QString &sessionRemoteTabTitleFormat = session()->tabTitleFormat(Session::RemoteTabTitle);
-    const QColor &sessionTabColor = session()->color();
+    const QString sessionLocalTabTitleFormat = session()->tabTitleFormat(Session::LocalTabTitle);
+    const QString sessionRemoteTabTitleFormat = session()->tabTitleFormat(Session::RemoteTabTitle);
+    const QColor sessionTabColor = session()->color();
 
-    QScopedPointer<RenameTabDialog> dialog(new RenameTabDialog(QApplication::activeWindow()));
+    auto *dialog = new RenameTabDialog(QApplication::activeWindow());
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->setModal(true);
     dialog->setTabTitleText(sessionLocalTabTitleFormat);
     dialog->setRemoteTabTitleText(sessionRemoteTabTitleFormat);
     dialog->setColor(sessionTabColor);
@@ -963,16 +944,10 @@ void SessionController::renameSession()
         dialog->focusTabTitleText();
     }
 
-    QPointer<Session> guard(session());
-    int result = dialog->exec();
-    if (guard.isNull()) {
-        return;
-    }
-
-    if (result != 0) {
-        const QString &tabTitle = dialog->tabTitleText();
-        const QString &remoteTabTitle = dialog->remoteTabTitleText();
-        const QColor &tabColor = dialog->color();
+    connect(dialog, &QDialog::accepted, this, [=]() {
+        const QString tabTitle = dialog->tabTitleText();
+        const QString remoteTabTitle = dialog->remoteTabTitleText();
+        const QColor tabColor = dialog->color();
 
         if (tabTitle != sessionLocalTabTitleFormat) {
             session()->setTabTitleFormat(Session::LocalTabTitle, tabTitle);
@@ -992,7 +967,9 @@ void SessionController::renameSession()
             Q_EMIT tabColoredByUser(true);
             snapshot();
         }
-    }
+    });
+
+    dialog->show();
 }
 
 // This is called upon Menu->Close Sesssion and right-click on tab->Close Tab
@@ -1198,7 +1175,9 @@ void SessionController::copyInputToSelectedTabs()
         _copyToGroup->setMasterMode(SessionGroup::CopyInputToAll);
     }
 
-    QPointer<CopyInputDialog> dialog = new CopyInputDialog(view());
+    auto *dialog = new CopyInputDialog(view());
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->setModal(true);
     dialog->setMasterSession(session());
 
     const QList<Session*> sessionsList = _copyToGroup->sessions();
@@ -1208,13 +1187,7 @@ void SessionController::copyInputToSelectedTabs()
 
     dialog->setChosenSessions(currentGroup);
 
-    QPointer<Session> guard(session());
-    int result = dialog->exec();
-    if (guard.isNull()) {
-        return;
-    }
-
-    if (result == QDialog::Accepted) {
+    connect(dialog, &QDialog::accepted, this, [=]() {
         QSet<Session*> newGroup = dialog->chosenSessions();
         newGroup.remove(session());
 
@@ -1231,7 +1204,9 @@ void SessionController::copyInputToSelectedTabs()
         _copyToGroup->setMasterMode(SessionGroup::CopyInputToAll);
         snapshot();
         Q_EMIT copyInputChanged(this);
-    }
+    });
+
+    dialog->show();
 }
 
 void SessionController::copyInputToNone()
@@ -1580,9 +1555,11 @@ void SessionController::changeSearchMatch()
 }
 void SessionController::showHistoryOptions()
 {
-    QScopedPointer<HistorySizeDialog> dialog(new HistorySizeDialog(QApplication::activeWindow()));
-    const HistoryType& currentHistory = session()->historyType();
+    auto *dialog = new HistorySizeDialog(QApplication::activeWindow());
+    dialog->setAttribute(Qt::WA_DeleteOnClose);
+    dialog->setModal(true);
 
+    const HistoryType &currentHistory = session()->historyType();
     if (currentHistory.isEnabled()) {
         if (currentHistory.isUnlimited()) {
             dialog->setMode(Enum::UnlimitedHistory);
@@ -1594,15 +1571,11 @@ void SessionController::showHistoryOptions()
         dialog->setMode(Enum::NoHistory);
     }
 
-    QPointer<Session> guard(session());
-    int result = dialog->exec();
-    if (guard.isNull()) {
-        return;
-    }
-
-    if (result != 0) {
+    connect(dialog, &QDialog::accepted, this, [this, dialog]() {
         scrollBackOptionsChanged(dialog->mode(), dialog->lineCount());
-    }
+    });
+
+    dialog->show();
 }
 void SessionController::sessionResizeRequest(const QSize& size)
 {
